/*
 * Copyright 2013 Akiban Technologies, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.akiban.jonquil;

import com.google.common.collect.HashMultimap;
import com.google.common.collect.Maps;
import com.google.common.collect.Multimap;
import com.google.common.collect.Multimaps;

import java.util.Collection;
import java.util.Map;

public final class JoinStrategyBuilder {

    public void addJoin(String parentTable, String parentField, String childTable, String childField) {
        Map.Entry<String, String> putKey = Maps.immutableEntry(parentTable, childTable);
        JoinFields join = new JoinFields(parentField, childField);
        joinsByTablePairing.put(putKey, join);
    }

    public JoinStrategy get() {
        return new ImmutableJoinStrategy(joinsByTablePairing);
    }

    private final Multimap<Map.Entry<String,String>, JoinFields> joinsByTablePairing = HashMultimap.create();

    private final class ImmutableJoinStrategy implements JoinStrategy {
        @Override
        public Collection<? extends JoinFields> getJoins(String parentTable, String childTable) {
            Map.Entry<String, String> lookupKey = Maps.immutableEntry(parentTable, childTable);
            Collection<? extends JoinFields> joins = joinsByTablePairing.get(lookupKey);
            if (joins == null)
                throw new JonquilException("no joins found between " + parentTable + " and " + childTable);
            return joins;
        }

        @Override
        public String toString() {
            return joinsByTablePairing.toString();
        }

        private ImmutableJoinStrategy(Multimap<Map.Entry<String, String>, JoinFields> joinsByTablePairing) {
            this.joinsByTablePairing = Multimaps.unmodifiableMultimap(joinsByTablePairing);
        }

        private final Multimap<Map.Entry<String,String>, JoinFields> joinsByTablePairing;
    }
}
