/*
 * Copyright 2013 Akiban Technologies, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.akiban.jonquil.actions;

import com.akiban.jonquil.JonquilException;

import java.util.HashMap;
import java.util.Map;

public final class Actions {

    public static Action findAction(String name) {
        name = name.toLowerCase();
        Action action = actionsMap.get(name);
        if (action == null)
            throw new JonquilException("unknown action: " + name);
        return action;
    }

    private Actions() {}

    private static final Map<String,Action> actionsMap = createActionsMap();

    private static Map<String, Action> createActionsMap() {
        Map<String, Action> actions = new HashMap<>();
        actions.put("limit", new Limit());
        actions.put("order_by", new OrderBy());
        return actions;
    }
}
