/*
 * Copyright 2013 Akiban Technologies, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.akiban.jonquil.actions;

import com.akiban.jonquil.JonquilException;
import com.akiban.jonquil.JonquilWriter;
import org.codehaus.jackson.JsonParser;
import org.codehaus.jackson.JsonToken;

import java.io.IOException;

@SuppressWarnings("unused")
final class OrderBy implements Action {
    @Override
    public void apply(JsonParser input, JonquilWriter output, String tableName) throws IOException {
        JsonToken token = input.nextToken();
        if (token == JsonToken.START_ARRAY) {
            for(token = input.nextToken(); token != JsonToken.END_ARRAY; token = input.nextToken()) {
                parseOrderByElem(input, output);
            }
        }
        else {
            parseOrderByElem(input, output);
        }
    }

    private void parseOrderByElem(JsonParser input, JonquilWriter output) throws IOException {
        JsonToken token = input.getCurrentToken();

        if (token == JsonToken.VALUE_STRING) {
            String field = input.getText();
            output.addOrderBy(field, JonquilWriter.Ordering.ASC);
        }
        else if (token == JsonToken.START_OBJECT) {
            if (input.nextToken() != JsonToken.FIELD_NAME)
                throw new JonquilException(BAD_ELEMENT_SPEC);
            String field = input.getText();
            if (input.nextToken() != JsonToken.VALUE_STRING)
                throw new JonquilException(BAD_ELEMENT_SPEC);
            String orderingString = input.getText();
            if (input.nextToken() != JsonToken.END_OBJECT)
                throw new JonquilException(BAD_ELEMENT_SPEC);
            JonquilWriter.Ordering ordering = JonquilWriter.Ordering.parse(orderingString);
            output.addOrderBy(field, ordering);
        }
        else {
            throw new JonquilException(BAD_ELEMENT_SPEC);
        }
    }

    private static final String BAD_ELEMENT_SPEC
            = "order-by elements must be a plain string or a one-element object of {\"field-name\" : \"asc|desc\"";
}
