/*
 * Copyright 2013 Akiban Technologies, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.akiban.jonquil;

import com.akiban.jonquil.actions.Action;
import com.google.common.base.Charsets;
import com.google.common.base.Functions;
import com.google.common.io.Resources;
import org.codehaus.jackson.JsonFactory;
import org.codehaus.jackson.JsonNode;
import org.codehaus.jackson.JsonParser;
import org.codehaus.jackson.JsonToken;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import static org.testng.AssertJUnit.assertEquals;

public final class JonquilTest {

    @DataProvider(name = "data")
    public Iterator<String[]> params() throws IOException {
        List<String[]> names = new ArrayList<>();
        try (InputStream is = resource(".")) {
            try (BufferedReader r = new BufferedReader(new InputStreamReader(is))) {
                for (String line; null != (line = r.readLine()); ) {
                    if ((line.length() > 1) && (line.charAt(0) != '_') && line.endsWith(".json")) {
                        String name = line.substring(0, line.length() - ".json".length());
                        names.add(new String[]{name});
                    }
                }
            }
        }
        assert ! names.isEmpty();
        return names.iterator();
    }

    @Test(dataProvider = "data")
    public void parse(String testName) throws IOException {
        JsonFactory factory = new JsonFactory();
        JsonParser parser = factory.createJsonParser(resource(testName + ".json"));
        Map<String, Action> map = new HashMap<>();
        map.put("fields", dummyAction);
        String actual = Jonquil.createSQL("customers", parser, joinStrategy, Functions.forMap(map, null));
        URL expectedURL = JonquilTest.class.getResource(testName + ".sql");
        String expected = Resources.toString(expectedURL, Charsets.UTF_8);
        assertEquals(expected, actual);
    }

    private static InputStream resource(String name) {
        return JonquilTest.class.getResourceAsStream(name);
    }

    private static final JoinStrategy joinStrategy = createJoinStrategy();

    private static JoinStrategy createJoinStrategy() {
        JsonNode joinDescription;
        try {
            joinDescription = JonquilUtils.jsonFactory.createJsonParser(resource("_joins.json")).readValueAsTree();
        } catch (IOException e) {
            throw new AssertionError(e);
        }
        return JonquilUtils.jsonBasedJoinStrategy(joinDescription);
    }

    private static Action dummyAction = new Action() {
        @Override
        public void apply(JsonParser input, JonquilWriter output, String tableName) throws IOException {
            if (input.nextToken() != JsonToken.START_ARRAY)
                throw new JonquilException("need array");
            while (input.nextToken() != JsonToken.END_ARRAY)
                output.addScalar(input.getText());
        }
    };
}
