Jonquil
=====

Jonqil is a simple, JSON-based query language designed for the Akiban Server. It compiles down to Akiban-dialect SQL. Jonquil stands for "Jonquil's object-notational query improvement language."

This project is still experimental -- or, if you prefer, a proof of concept. See the end of the readme for where the idea could evolve.

To provide feedback (ideas, bug reports, etc), you can use the standard github stuff or drop by our IRC channel, #akiban on freenode.net.

Examples
========

This bit of JSON:

```json
{
    "cid" : [],
    "last_name" : "Smith",
    "orders" : {
        "@limit" : 5,
        "@order_by" : {"placed" : "descending"},
        "placed" : ["2013-01-02", "2013-03-07"],
        "items" : {
            "cost" : "=quantity * cost_per_item"
        }
    }
}
```

produces this SQL query (to which I've added some newlines and indentation):

```sql
SELECT "cid", "last_name",
    (SELECT "placed",
        (SELECT (quantity * cost_per_item) "cost"
         FROM "items"
         WHERE "orders"."oid1" = "items"."o_id1"
         AND "orders"."oid2" = "items"."o_id2") "items"
     FROM "orders"
     WHERE "placed" IN('2013-01-02','2013-03-07')
     AND "customers"."cid" = "orders"."c_id"
     ORDER BY "placed" DESC
     LIMIT 5) "orders"
FROM "customers"
WHERE "last_name"='Smith';
```

This is a bit of weird SQL syntax! In standard SQL, a subselect may only return one value -- a single row with a single field in it. But since Akiban is all about hierarchical data, we let you have richer subselects. In our standard mode (where we output as tables), we still require just a single row; but you can also set the connection to output as JSON, in which case the above SQL does what you'd expect it to do:

- get all customers whose last name is "Smith"
- for each customer, get five of their orders
- for each order, get its line items
- for each line item, calculate its cost

A bit more formally
===================
(...but not anything as formal as BNF)

- a select-fragment is a JSON object
    - the top-level element of a Jonquil query is a select-fragment
- most keys in a select-fragment specify the name of a field
- keys that start with '@' do not specify a field; they specify actions (more on this in a bit)
- all fields specified in a select-fragment will be outputted
- if a field's value is:
    - a scalar value, then there will be a `WHERE` condition on this field saying it must equal the value
    - an empty array, then there is no `WHERE` condition added (ie, this just selects)
    - a non-empty array, then there will be a `WHERE` condition on this field saying it must be `IN` the list of values specified.
    - an object, then the field name is taken to be a table that's grouped under the current table, and the object represents a select-fragment which will form a sub-select

Each action defines its own values.
- Right now, the only built-in actions are:
    - `@limit`, which takes an integer
    - `@order_by`, which takes either a single orderby-element or an array of orderby-elements. Each orderby-element is either a string representing a field name to be ordered ascending, or a map with a single entry, `{<field_name> : "asc|desc"}`
- The Akiban Server adds in its own action called `@fields`. If the value is "all", then this adds (without conditions) all fields for the table that this select-fragment targets. Otherwise, the value must be an array of strings specifying the name of fields to add.

Compiling and running
=====================

Nothing special to it, just the standard `mvn` stuff:

    mvn package

The `pom.xml` is set up to deploy to Akiban's private repos, so unless you have our repo's password, you won't be able to run `mvn deploy`. If you are not affiliated with Akiban and do have our repo's password, please let us know so that we can change it. You also won't be able to use a Jonquil as a dependency for another project out of the box
unless you manually run `mvn install` on each machine that's compilng that other project, so that that machine has Jonquil in its local repo.

The `mvn package` task will create a `jonquil-<version>-jar-with-dependencies.jar` file, which you can run by invoking `java -jar target/*-jar-with-dependencies.jar`. This will provide you a very simple skelton of a schema (basically just the table names with their PKs and FKs), and puts you into an endless loop in which you type in an Jonquil object and it spits out the corresponding SQL.

Entry point
==========
`com.akiban.jonquil.Jonquil.createSQL`

To the future!
==============

I have a few ideas for future work:
- more actions
- ability to specify join conditions for sub-selects (right now, they're provided by the application when it passes a `JoinStrategy` to `Jonquil.createSQL`)
- ability to specify grouping, or possibly automatically group by PKs, so that aggregates can work
- ability to reference a field in one select-fragment in the one above it. For instance, in the example above, imagining `orders` being able to specify an entry `"order_total" : "=sum(line_items.cost)"`

If you have any other ideas, don't be a stranger.
